using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;
using System.Windows.Forms;
using NET.MegaGrid;
using NET.MegaGrid.EArgs;

namespace Demo
{
	public partial class EditForm : Form
	{
		[DllImport("shlwapi.dll", CharSet = CharSet.Auto)]
		static extern bool PathRelativePathTo(
			 [Out] StringBuilder pszPath,
			 [In] string pszFrom,
			 [In] int dwAttrFrom,
			 [In] string pszTo,
			 [In] int dwAttrTo
		);

		private int MaxTag;
		private int MaxID;
		private string sFileName;
		private string sHeaderRow;
		private string sLocDir;
		private ArrayList rowsArr = new ArrayList();
		private ArrayList N_rowsArr = new ArrayList();
		private ArrayList E_rowsArr = new ArrayList();
		private string[] CountriesArray = new string[6] { "Canada", "Denmark", "Finland", "Norway", "Russia", "USA" };
		private string[] N_CountriesArray = new string[2] { "Canada", "USA" };
		private string[] E_CountriesArray = new string[4] { "Denmark", "Finland", "Norway", "Russia"};

		public EditForm()
		{
			InitializeComponent();
		}

		private void ConfigureLang()
		{
			int iLangOption = cbxLangs.SelectedIndex;
			if (iLangOption == 0)
				MegaGridCtrl.LanguageOption = enumLanguage.Chinese_Simplified;
			else if (iLangOption == 1)
				MegaGridCtrl.LanguageOption = enumLanguage.Chinese_Traditional;
			else if (iLangOption == 2)
				MegaGridCtrl.LanguageOption = enumLanguage.English;
			else if (iLangOption == 3)
				MegaGridCtrl.LanguageOption = enumLanguage.French;
			else if (iLangOption == 4)
				MegaGridCtrl.LanguageOption = enumLanguage.German;
			else if (iLangOption == 5)
				MegaGridCtrl.LanguageOption = enumLanguage.Italian;
			else if (iLangOption == 6)
				MegaGridCtrl.LanguageOption = enumLanguage.Japanese;
			else if (iLangOption == 7)
				MegaGridCtrl.LanguageOption = enumLanguage.Korean;
			else if (iLangOption == 8)
				MegaGridCtrl.LanguageOption = enumLanguage.Portuguese;
			else if (iLangOption == 9)
				MegaGridCtrl.LanguageOption = enumLanguage.Russian;
			else
				MegaGridCtrl.LanguageOption = enumLanguage.Spanish;
		}

		private void MegaGridEdit_Load(object sender, EventArgs e)
		{
			cbxEditModes.SelectedIndex	= 0;
			cbxLangs.SelectedIndex		= 2;
			cbxGridLayout.SelectedIndex = 0;
		}

		private void cbxLangs_SelectedIndexChanged(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void cbxEditModes_SelectedIndexChanged(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void chkAllowSections_CheckedChanged(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void cbxGridLayout_SelectedIndexChanged(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void PopulateData()
		{
			//Read the data from a file
			sLocDir = System.AppDomain.CurrentDomain.BaseDirectory;
			sFileName = Path.GetFullPath(Path.Combine(sLocDir, "..\\..\\misc\\")) + "EditCities.txt";
			string[] readText = File.ReadAllLines(sFileName, Encoding.UTF8);
			int counter = 0;
			rowsArr = new ArrayList();
			N_rowsArr = new ArrayList();
			E_rowsArr = new ArrayList();

			MaxID = -1;
			foreach (string line in readText)
			{
				if (counter == 0)
				{
					sHeaderRow = line;
				}
				else
				{
					string[] cells = line.Split('|');
					EditRow objRow = new EditRow();

					if (cells.Length == 9)
					{
						objRow.Tag = MaxTag;
						MaxTag = MaxTag + 1;

						objRow.ID = Convert.ToInt16(cells[0]);
						if (objRow.ID > MaxID)
							MaxID = objRow.ID;

						objRow.Latitude = cells[1].Trim();
						objRow.Longitude = cells[2].Trim();
						objRow.City = cells[3].Trim();
						objRow.ProvState = cells[4].Trim();
						objRow.Country = cells[5];

						try
						{
							string sImageFilePath = cells[6].Trim();
							if (sImageFilePath != "")
								objRow.NationalFlagImage = Path.GetFullPath(sImageFilePath);
						}
						catch (Exception ex)
						{
							objRow.NationalFlagImage = "";
						}
						objRow.IsNorthAmerica = cells[7].Trim();
						objRow.LastUpdatedDate = cells[8].Trim();

						rowsArr.Add(objRow);
					}
				}

				counter++;
			}

			//Edit Grid
			MegaGridCtrl.ReadOnly = false;

			//Edit Mode
			MegaGridCtrl.EditMode = (enumEditMode)(cbxEditModes.SelectedIndex + 1);

			//Configure language
			ConfigureLang();

			//Configure grid layout
			ConfigureGridLayout.SetLayout(MegaGridCtrl, cbxGridLayout.SelectedIndex);

			if (chkAllowSections.Checked)
			{
				MegaGridCtrl.AllowSections		= true;
				MegaGridCtrl.AllowSectionIcon	= true;
				
				MegaGridCtrl.SectionIconHeight = 16;
				MegaGridCtrl.SectionIconWidth = 16;

				//Find the number of rows for North American and Europe respectively
				int NCount = 0;
				int ECount = 0;
				for (int i = 0; i < rowsArr.Count; i++)
				{
					EditRow objRow = (EditRow)rowsArr[i];
					if (objRow.IsNorthAmerica.CompareTo("YES") == 0)
						NCount++;
					else
						ECount++;
				}
				//Rows in North America
				MegaGridCtrl.Sections[0].RowsCount = NCount;
				MegaGridCtrl.Sections[0].SectionImage = Image.FromFile(Path.GetFullPath(Path.Combine(sLocDir, "..\\..\\misc\\")) + "Icon_N.png");

				//Rows in Europe
				MegaGridCtrl.Sections[1].RowsCount = ECount;
				MegaGridCtrl.Sections[1].SectionImage = Image.FromFile(Path.GetFullPath(Path.Combine(sLocDir, "..\\..\\misc\\")) + "Icon_E.png");

			}
			else
			{
				MegaGridCtrl.AllowSections = false;

				//Set Rows Count
				MegaGridCtrl.RowsCount = rowsArr.Count;
			}

			//Configure MaskEdit TextBox for Latitude
			MegaGridCtrl.ConfigureCellMaskedTextBox(-1,
													1,
													true,
													MaskFormat.IncludeLiterals,
													false,
													true,
													"0000'N",
													'\0',
													'_',
													false,
													false,
													true,
													RightToLeft.No,
													HorizontalAlignment.Left,
													MaskFormat.IncludeLiterals,
													false);

			//Configure MaskEdit TextBox for Longitude
			MegaGridCtrl.ConfigureCellMaskedTextBox(-1,
													2,
													true,
													MaskFormat.IncludeLiterals,
													false,
													true,
													"0000'?",
													'\0',
													'_',
													false,
													false,
													true,
													RightToLeft.No,
													HorizontalAlignment.Left,
													MaskFormat.IncludeLiterals,
													false);

			//Configure countries' ComboBox
			if (chkAllowSections.Checked)
			{
				//Configure North America
				ArrayList N_CountriesList = new ArrayList();
				N_CountriesList.AddRange(N_CountriesArray);
				MegaGridCtrl.ConfigureCellComboBox(0, 5, N_CountriesList);

				//Configure Europe
				ArrayList E_CountriesList = new ArrayList();
				E_CountriesList.AddRange(E_CountriesArray);
				MegaGridCtrl.ConfigureCellComboBox(1, 5, E_CountriesList);
			}
			else
			{
				//No sections
				ArrayList CountriesList = new ArrayList();
				CountriesList.AddRange(CountriesArray);
				MegaGridCtrl.ConfigureCellComboBox(-1, 5, CountriesList);
			}

			//Configure DateTimePicker for Last Updated Date's Column
			DateTime maxDate = new DateTime(3000, 12, 1);
			DateTime minDate = new DateTime(2000, 12, 1);
			MegaGridCtrl.ConfigureCellDatePicker(-1, 8, maxDate, minDate, DateTimePickerFormat.Custom, "yyyy-MM-dd");

			//Alert: The following 2 lines of code are very IMPORTANT and mandatory for Grid in edit mode 
			//to work properly.
			//Configure main grid and all sub grids
			MegaGridCtrl.ConfigureAllGrids();

			//Update edit-mode columns' settings for the full grid and all sub grids
			MegaGridCtrl.UpdateEditColumns();

			if (MegaGridCtrl.RowsCount == 0)
			{
				MegaGridCtrl.ClearGrid();
				return;
			}

			int NIndex = 0;
			int EIndex = 0;
			int SectionIndex = 0;
			int RowIndex = 0;
			for (int i = 0; i <= (rowsArr.Count - 1); i++)
			{
				EditRow objRow = (EditRow)rowsArr[i];

				if (chkAllowSections.Checked)
				{
					if (objRow.IsNorthAmerica.CompareTo("YES") == 0)
					{
						//North America
						N_rowsArr.Add(objRow);
						SectionIndex = 0;
						RowIndex = NIndex;
					}
					else
					{
						//Europe
						E_rowsArr.Add(objRow);
						SectionIndex = 1;
						RowIndex = EIndex;
					}
				}
				else
				{
					SectionIndex = -1;
					RowIndex = i;
				}

				MegaGridCtrl.ConfigureEditRow(SectionIndex, RowIndex, objRow.Tag);

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 0, Convert.ToString(objRow.ID));

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 1, objRow.Latitude);

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 2, objRow.Longitude);

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 3, objRow.City);

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 4, objRow.ProvState);

				if (SectionIndex == -1)
					MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(CountriesArray, objRow.Country)));
				else if (SectionIndex == 0)
					MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(N_CountriesArray, objRow.Country)));
				else
					MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(E_CountriesArray, objRow.Country)));

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 6, objRow.NationalFlagImage);

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 7, objRow.IsNorthAmerica.ToUpper());

				MegaGridCtrl.ConfigureEditCell(SectionIndex, RowIndex, 8, objRow.LastUpdatedDate);

				if (chkAllowSections.Checked)
				{
					if (objRow.IsNorthAmerica.CompareTo("YES") == 0)
						NIndex++;
					else
						EIndex++;
				}
			}
		}

		private int GetCountryIndex(string[] CArray, string CountryName)
		{
			for (int i = 0; i < CArray.Length; i++)
			{
				if (CountryName == CArray[i])
					return i;
			}

			return -1;
		}

		private bool IsValidMaskedValue(string sOrigStr)
		{
			//Check if a string is a valid Latitude or Longitude
			char Char_1 = Convert.ToChar(sOrigStr.Substring(0, 1));
			char Char_2 = Convert.ToChar(sOrigStr.Substring(1, 1));
			char Char_3 = Convert.ToChar(sOrigStr.Substring(3, 1));
			char Char_4 = Convert.ToChar(sOrigStr.Substring(4, 1));

			return (Char.IsDigit(Char_1) &&
					Char.IsDigit(Char_2) &&
					Char.IsDigit(Char_3) &&
					Char.IsDigit(Char_4));
		}

		private string CheckError(EventEditGridRow objEEGR)
		{
			string sErrorMsg;
			ArrayList cellsArr;
			string sCellValue;
			DateTime LastUpdatedDate;

			sErrorMsg = "";
			cellsArr = objEEGR.CellsList;
			int columnsCount = cellsArr.Count;
			for (int i = 0; i <= (columnsCount - 1); i++)
			{
				EventEditGridCell objEditCell = (EventEditGridCell)cellsArr[i];
				sCellValue = objEditCell.NewValue;

				if ((objEditCell.ColumnName == "Latitude") && (objEditCell.OrigColumnIndex == 1))
				{
					//Check if Latitude value is valid or not
					if (!IsValidMaskedValue(sCellValue))
						sErrorMsg += sCellValue + " is not a valid latitude value." + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "Longitude") && (objEditCell.OrigColumnIndex == 2))
				{
					//Check if Longitude value is valid or not
					Char sLastChar = Convert.ToChar(sCellValue.Substring(sCellValue.Length - 1, 1));
					if ((!IsValidMaskedValue(sCellValue)) ||
						((Convert.ToChar(sLastChar) != 'E') && (Convert.ToChar(sLastChar) != 'W')))
						sErrorMsg += sCellValue + " is not a valid longitude value." + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "City") && (objEditCell.OrigColumnIndex == 3))
				{
					//Check if City is empty or not
					if (sCellValue.Trim().Length == 0)
						sErrorMsg += "City is empty." + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "Province/State") && (objEditCell.OrigColumnIndex == 4))
				{
					//Check if Province/State is empty or not
					if (sCellValue.Trim().Length == 0)
						sErrorMsg += "Province/State is empty." + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "Country") && (objEditCell.OrigColumnIndex == 5))
				{
					//Check if Country is valid or not
					if (sCellValue.Trim().Length == 0)
						sErrorMsg += "Country is empty." + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "North America?") && (objEditCell.OrigColumnIndex == 7))
				{
					//Check continent
					string locSubMsg = CheckContinent(objEEGR, sCellValue.Trim());
					if (locSubMsg != "")
						sErrorMsg += locSubMsg + Environment.NewLine;
				}
				else if ((objEditCell.ColumnName == "Last Updated Date") && (objEditCell.OrigColumnIndex == 8))
				{
					//Check if Last Updated Date is valid or not
					if (sCellValue == "")
					{
						sErrorMsg += "Last Updated Date was not specified" + Environment.NewLine;
					}
					else
					{
						try
						{
							LastUpdatedDate = DateTime.ParseExact(sCellValue, "yyyy-MM-dd", null);
							if (LastUpdatedDate > DateTime.Now)
								sErrorMsg += "Last Updated Date can not be greater than today." + Environment.NewLine;
						}
						catch (Exception ex)
						{
						}
					}
				}
			}

			return sErrorMsg;
		}

		private string CheckContinent(EventEditGridRow objEEGR, string sCellValue)
		{
			if (objEEGR.GridIndex == -1)
			{
				if (((objEEGR.SectionIndex == 0) && (sCellValue == "NO")) ||
					((objEEGR.SectionIndex == 1) && (sCellValue == "YES")))
					return "Invalid value for \"In North America?\"";
			}
			else
			{
				if (chkAllowSections.Checked)
				{
					if (((objEEGR.GridIndex == 0) && (sCellValue == "NO")) ||
						((objEEGR.GridIndex == 1) && (sCellValue == "YES")))
						return "Invalid value for \"In North America?\"";
				}
			}

			return "";
		}

		private void SetCountryName(EventEditGridRow objEEGR, EditRow objRow, string sCellValue)
		{
			if (objEEGR.GridIndex == -1)
			{
				//In full grid
				if (objEEGR.SectionIndex == -1)
					//No Section
					objRow.Country = CountriesArray[Convert.ToInt16(sCellValue)];
				else if (objEEGR.SectionIndex == 0)
					//Section for North America
					objRow.Country = N_CountriesArray[Convert.ToInt16(sCellValue)];
				else
					//Section for Europe
					objRow.Country = E_CountriesArray[Convert.ToInt16(sCellValue)];
			}
			else
			{
				if (chkAllowSections.Checked)
				{
					//With Sections
					if (objEEGR.GridIndex == 0)
						//Sub grid for North America
						objRow.Country = N_CountriesArray[Convert.ToInt16(sCellValue)];
					else
						//Sub grid for Europe
						objRow.Country = E_CountriesArray[Convert.ToInt16(sCellValue)];
				}
				else
				{
					//No Sections
					objRow.Country = CountriesArray[Convert.ToInt16(sCellValue)];
				}
			}
		}

		private void FillEditRow(EventEditGridRow objEEGR, EditRow objRow)
		{
			string sCellValue;
			ArrayList cellsArr = objEEGR.CellsList;
			int columnsCount = cellsArr.Count;
			for (int i = 0; i <= (columnsCount - 1); i++)
			{
				EventEditGridCell objEditCell = (EventEditGridCell)cellsArr[i];
				sCellValue = objEditCell.NewValue;

				//Set field values
				if ((objEditCell.ColumnName == "Latitude") && (objEditCell.OrigColumnIndex == 1))
					//Latitude value
					objRow.Latitude = sCellValue;
				else if ((objEditCell.ColumnName == "Longitude") && (objEditCell.OrigColumnIndex == 2))
					//Longitude value
					objRow.Longitude = sCellValue;
				else if ((objEditCell.ColumnName == "City") && (objEditCell.OrigColumnIndex == 3))
					//City
					objRow.City = sCellValue;
				else if ((objEditCell.ColumnName == "Province/State") && (objEditCell.OrigColumnIndex == 4))
					//Province/State
					objRow.ProvState = sCellValue;
				else if ((objEditCell.ColumnName == "Country") && (objEditCell.OrigColumnIndex == 5))
					//Country
					SetCountryName(objEEGR, objRow, sCellValue);
				else if ((objEditCell.ColumnName == "National Flag") && (objEditCell.OrigColumnIndex == 6))
					//National Flag Image
					objRow.NationalFlagImage = sCellValue;
				else if ((objEditCell.ColumnName == "North America?") && (objEditCell.OrigColumnIndex == 7))
					//In North America?
					objRow.IsNorthAmerica = sCellValue;
				else if ((objEditCell.ColumnName == "Last Updated Date") && (objEditCell.OrigColumnIndex == 8))
					//Last Updated Date
					objRow.LastUpdatedDate = sCellValue;
			}
		}

		private string RemoveBars(String sOrigStr)
		{
			return sOrigStr.Replace("|", "");
		}

		private void CollectRowsInList(ArrayList editRows, ref string sLines)
		{
			const int MAX_PATH = 256;
			String sSingleLine = "";

			for (int i = 0; i <= (editRows.Count - 1); i++)
			{
				EditRow objRow = (EditRow)editRows[i];

				//Add a row
				sSingleLine = Convert.ToString(objRow.ID);
				sSingleLine = sSingleLine + "|" + objRow.Latitude;
				sSingleLine = sSingleLine + "|" + objRow.Longitude;
				sSingleLine = sSingleLine + "|" + RemoveBars(objRow.City);
				sSingleLine = sSingleLine + "|" + RemoveBars(objRow.ProvState);
				sSingleLine = sSingleLine + "|" + objRow.Country;

				//Generate the relative path for an image file
				try
				{
					StringBuilder pszPath = new StringBuilder(MAX_PATH);
					String pszFrom = sLocDir;
					PathRelativePathTo(pszPath,
										pszFrom,
										(int)FileAttributes.Directory,
										objRow.NationalFlagImage,
										(int)FileAttributes.Normal);

					sSingleLine = sSingleLine + "|" + pszPath;
				}
				catch (Exception ex)
				{
					sSingleLine = sSingleLine + "|" + objRow.NationalFlagImage;
				}

				sSingleLine = sSingleLine + "|" + objRow.IsNorthAmerica;
				sSingleLine = sSingleLine + "|" + objRow.LastUpdatedDate + Environment.NewLine;

				sLines = sLines + sSingleLine;
			}
		}

		private void SaveRows()
		{
			//Header
			string sLines = sHeaderRow + Environment.NewLine;

			if (chkAllowSections.Checked)
			{
				//Collect all rows for North America
				CollectRowsInList(N_rowsArr, ref sLines);

				//Collect all rows for Europe
				CollectRowsInList(E_rowsArr, ref sLines);
			}
			else
			{
				//Plain grid
				CollectRowsInList(rowsArr, ref sLines);
			}

			try
			{
				//Save all
				File.WriteAllText(sFileName, sLines);
			}
			catch (Exception ex)
			{
			}
		}

		private EditRow SearchRowInList(ArrayList editRows, object Tag, ref int RowPos)
		{
			int iCount = editRows.Count;
			for (int i = 0; i <= (iCount - 1); i++)
			{
				EditRow objRow = (EditRow)editRows[i];
				if (Convert.ToString(objRow.Tag) == Convert.ToString(Tag))
				{
					RowPos = i;
					return objRow;
				}
			}

			RowPos = -1;
			return null;
		}

		private EditRow SearchRow(EventEditGridRow objEEGR, ref int RowPos)
		{
			if (chkAllowSections.Checked)
			{
				ArrayList editRows;
				if (objEEGR.GridIndex == -1)
				{
					//In full grid
					if (objEEGR.SectionIndex == 0)
						editRows = N_rowsArr;
					else
						editRows = E_rowsArr;
				}
				else
				{
					//In a sub grid
					if (objEEGR.GridIndex == 0)
						editRows = N_rowsArr;
					else
						editRows = E_rowsArr;
				}

				//Conduct search
				return SearchRowInList(editRows, objEEGR.Tag, ref RowPos);
			}
			else
			{
				//No sections (plain grid)
				return SearchRowInList(rowsArr, objEEGR.Tag, ref RowPos);
			}
		}

		private bool AddRow(EventEditGridRow objEEGR)
		{
			//Insert a new record
			if (objEEGR.OnBorder)
				objEEGR.UseNextCopyGrid = true;

			ArrayList cellsArr = objEEGR.CellsList;
			int columnsCount = cellsArr.Count;
			for (int i = 0; i <= (columnsCount - 1); i++)
			{
				EventEditGridCell objEditCell = (EventEditGridCell)cellsArr[i];
				if ((objEditCell.ColumnName == "ID") && (objEditCell.OrigColumnIndex == 0))
					objEditCell.NewValue = Convert.ToString(MaxID + 1);
			}

			EditRow newEditRow = new EditRow();
			FillEditRow(objEEGR, newEditRow);

			//Set ID
			newEditRow.ID = MaxID + 1;
			MaxID = MaxID + 1;

			//Set tags
			newEditRow.Tag = MaxTag;
			objEEGR.Tag = MaxTag;

			if (chkAllowSections.Checked)
			{
				if (objEEGR.GridIndex == -1)
				{
					//In Full Grid
					if (objEEGR.SectionIndex == 0)
					{
						//North America's Section
						if ((objEEGR.MainRowIndex >= 0) && (objEEGR.MainRowIndex < N_rowsArr.Count))
							N_rowsArr.Insert((int)objEEGR.MainRowIndex, newEditRow);
						else
							N_rowsArr.Add(newEditRow);
					}
					else
					{
						//Europe's Section
						if ((objEEGR.MainRowIndex >= 0) && (objEEGR.MainRowIndex < E_rowsArr.Count))
							E_rowsArr.Insert((int)objEEGR.MainRowIndex, newEditRow);
						else
							E_rowsArr.Add(newEditRow);
					}
				}
				else if (objEEGR.GridIndex == 0)
				{
					//Sub Grid for North America
					if ((objEEGR.RowIndex >= 0) && (objEEGR.RowIndex < N_rowsArr.Count))
						N_rowsArr.Insert((int)objEEGR.RowIndex, newEditRow);
					else
						N_rowsArr.Add(newEditRow);
				}
				else
				{
					//Sub Grid for Europe
					if ((objEEGR.RowIndex >= 0) && (objEEGR.RowIndex < E_rowsArr.Count))
						E_rowsArr.Insert((int)objEEGR.RowIndex, newEditRow);
					else
						E_rowsArr.Add(newEditRow);
				}
			}
			else
			{
				//Plain grid without section(s)
				if ((objEEGR.MainRowIndex >= 0) && (objEEGR.MainRowIndex < rowsArr.Count))
					rowsArr.Insert((int)objEEGR.MainRowIndex, newEditRow);
				else
					rowsArr.Add(newEditRow);
			}

			SaveRows();

			//Bump up MaxTag by 1
			MaxTag = MaxTag + 1;

			return true;
		}

		private bool UpdateRow(EventEditGridRow objEEGR)
		{
			int iRowPos = 0;
			EditRow objRow = SearchRow(objEEGR, ref iRowPos);

			if (iRowPos == -1)
				return false;
			else
			{
				FillEditRow(objEEGR, objRow);
				SaveRows();
				return true;
			}
		}

		private bool DeleteRow(EventEditGridRow objEEGR)
		{
			int iRowPos = 0;
			EditRow objRow = SearchRow(objEEGR, ref iRowPos);

			if (iRowPos == -1)
				return false;
			else
			{
				ArrayList editRows = rowsArr;
				if (chkAllowSections.Checked)
				{
					if (objEEGR.GridIndex == -1)
					{
						//In full grid
						if (objEEGR.SectionIndex == 0)
							editRows = N_rowsArr;
						else
							editRows = E_rowsArr;
					}
					else
					{
						//In a sub grid
						if (objEEGR.GridIndex == 0)
							editRows = N_rowsArr;
						else
							editRows = E_rowsArr;
					}
				}
				editRows.RemoveAt(iRowPos);

				SaveRows();
				return true;
			}
		}

		private void MegaGridEdit_MGEditChangeRows(object s, EditChangeRowsEventArgs e)
		{
			int count;
			string sSingleErrorMsg;

			count = e.RowsList.Count;
			for (int i = 0; i <= (count - 1); i++)
			{
				EventEditGridRow objEEGR = (EventEditGridRow)e.RowsList[i];

				String sRowFlag = "Row with index " + Convert.ToString(objEEGR.RowIndex);
				String sComboFlag = "Row with index " + Convert.ToString(objEEGR.RowIndex) + " and tag " + Convert.ToString(objEEGR.Tag);
				if ((objEEGR.Flag == 1) || (objEEGR.Flag == 2))
				{
					//Add or update a row
					sSingleErrorMsg = CheckError(objEEGR);
					if (sSingleErrorMsg != "")
					{
						objEEGR.Success = false;
						if (objEEGR.Flag == 1)
							objEEGR.Message = sRowFlag + " - " + sSingleErrorMsg;
						else
							objEEGR.Message = sComboFlag + " - " + sSingleErrorMsg;
					}
					else
					{
						if (objEEGR.Flag == 1)
						{
							//Add a row
							AddRow(objEEGR);
							objEEGR.Success = true;
						}
						else
						{
							//Update a row
							if (!UpdateRow(objEEGR))
							{
								objEEGR.Success = false;
								objEEGR.Message = sComboFlag + " - Can't update current row due to invalid row tag";
							}
							else
							{
								objEEGR.Success = true;
							}
						}
					}
				}
				else
				{
					//Delete a row
					if (!DeleteRow(objEEGR))
					{
						objEEGR.Success = false;
						objEEGR.Message = sComboFlag  + " - Can't delete current row due to invalid row tag";
					}
					else
					{
						objEEGR.Success = true;
					}
				}
			}

			e.Done = true;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			this.Close();
		}
	}
}