Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.IO
Imports System.Runtime
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports NET.MegaGrid
Imports NET.MegaGrid.EArgs

Public Class EditModeForm

    <DllImport("shlwapi.dll", CharSet:=CharSet.Auto)> _
    Public Shared Function PathRelativePathTo( _
         ByVal pszPath As StringBuilder, _
         ByVal pszFrom As String, _
         ByVal dwAttrFrom As Integer, _
         ByVal pszTo As String, _
         ByVal dwAttrTo As Integer) As Boolean
    End Function

    Private MaxTag As Integer
    Private MaxID As Integer
    Private rowsArr As New ArrayList()
    Private N_rowsArr As New ArrayList()
    Private E_rowsArr As New ArrayList()

    Private sFileName As String
    Private sLocDir As String
    Private sHeaderRow As String
    Private CountriesArray() As String = {"Canada", "Denmark", "Finland", "Norway", "Russia", "USA"}
    Private N_CountriesArray() As String = {"Canada", "USA"}
    Private E_CountriesArray() As String = {"Denmark", "Finland", "Norway", "Russia"}


    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        Me.Close()
    End Sub

    Private Sub EditModeForm_Shown(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Shown
        sLocDir = System.AppDomain.CurrentDomain.BaseDirectory
        sFileName = Path.GetFullPath(Path.Combine(sLocDir, "..\..\misc\")) + "EditCities.txt"

        cbxEditModes.SelectedIndex = 0
        cbxLangs.SelectedIndex = 2
        cbxGridLayout.SelectedIndex = 0
    End Sub

    Private Sub cbxEditModes_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxEditModes.SelectedIndexChanged
        PopulateData()
    End Sub

    Private Sub cbxLangs_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxLangs.SelectedIndexChanged
        PopulateData()
    End Sub

    Private Sub chkAllowSections_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowSections.CheckedChanged
        PopulateData()
    End Sub

    Private Sub cbxGridLayout_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxGridLayout.SelectedIndexChanged
        PopulateData()
    End Sub

    Private Sub ConfigureLang()
        Dim iLangOption As Integer = cbxLangs.SelectedIndex
        If (iLangOption = 0) Then
            MegaGridEdit.LanguageOption = enumLanguage.Chinese_Simplified
        ElseIf (iLangOption = 1) Then
            MegaGridEdit.LanguageOption = enumLanguage.Chinese_Traditional
        ElseIf (iLangOption = 2) Then
            MegaGridEdit.LanguageOption = enumLanguage.English
        ElseIf (iLangOption = 3) Then
            MegaGridEdit.LanguageOption = enumLanguage.French
        ElseIf (iLangOption = 4) Then
            MegaGridEdit.LanguageOption = enumLanguage.German
        ElseIf (iLangOption = 5) Then
            MegaGridEdit.LanguageOption = enumLanguage.Italian
        ElseIf (iLangOption = 6) Then
            MegaGridEdit.LanguageOption = enumLanguage.Japanese
        ElseIf (iLangOption = 7) Then
            MegaGridEdit.LanguageOption = enumLanguage.Korean
        ElseIf (iLangOption = 8) Then
            MegaGridEdit.LanguageOption = enumLanguage.Portuguese
        ElseIf (iLangOption = 9) Then
            MegaGridEdit.LanguageOption = enumLanguage.Russian
        Else
            MegaGridEdit.LanguageOption = enumLanguage.Spanish
        End If
    End Sub

    Private Sub PopulateData()
        Dim locDir As [String] = System.AppDomain.CurrentDomain.BaseDirectory
        locDir = Path.GetFullPath(Path.Combine(locDir, "..\..\misc\"))

        'Read the data from a file
        Dim readText As String() = File.ReadAllLines(sFileName, Encoding.UTF8)
        Dim counter As Integer = 0
        Dim I As Integer

        rowsArr = New ArrayList()
        N_rowsArr = New ArrayList()
        E_rowsArr = New ArrayList()

        MaxTag = 100
        MaxID = 0
        For Each line As String In readText

            If (counter = 0) Then
                sHeaderRow = line
            Else
                Dim cells As String() = line.Split("|"c)
                Dim objRow As New EditRow()

                If (cells.Length = 9) Then
                    objRow.Tag = MaxTag
                    MaxTag = MaxTag + 1

                    objRow.ID = Convert.ToInt16(cells(0))
                    If (objRow.ID > MaxID) Then
                        MaxID = objRow.ID
                    End If

                    objRow.Latitude = cells(1).Trim()
                    objRow.Longitude = cells(2).Trim()
                    objRow.City = cells(3).Trim()
                    objRow.ProvState = cells(4).Trim()
                    objRow.Country = cells(5)

                    Try
                        Dim sImageFilePath = cells(6).Trim()
                        If (sImageFilePath <> "") Then
                            objRow.NationalFlagImage = Path.GetFullPath(sImageFilePath)
                        End If
                    Catch ex As Exception
                        objRow.NationalFlagImage = ""
                    End Try

                    objRow.IsNorthAmerica = cells(7).Trim()
                    objRow.LastUpdatedDate = cells(8).Trim()

                    rowsArr.Add(objRow)
                End If
            End If

            counter = counter + 1
        Next

        'Edit Grid
        MegaGridEdit.ReadOnly = False

        'Which Edit Mode?
        MegaGridEdit.EditMode = cbxEditModes.SelectedIndex + 1

        'Language
        ConfigureLang()

        'Grid layout
        ConfigureGridLayout.SetLayout(MegaGridEdit, cbxGridLayout.SelectedIndex)

        If (chkAllowSections.Checked) Then
            MegaGridEdit.AllowSections = True
            MegaGridEdit.AllowSectionIcon = True

            MegaGridEdit.SectionIconHeight = 16
            MegaGridEdit.SectionIconWidth = 16

            'Find the number of rows for North American
            Dim NCount As Integer = 0
            Dim ECount As Integer = 0
            For I = 0 To (rowsArr.Count - 1)
                Dim objRow As EditRow = rowsArr(I)
                If (objRow.IsNorthAmerica.CompareTo("YES") = 0) Then
                    NCount = NCount + 1
                Else
                    ECount = ECount + 1
                End If
            Next I

            MegaGridEdit.Sections(0).RowsCount = NCount
            MegaGridEdit.Sections(0).SectionImage = Image.FromFile(Path.GetFullPath(Path.Combine(sLocDir, "..\\..\\misc\\")) + "Icon_N.png")

            MegaGridEdit.Sections(1).RowsCount = ECount
            MegaGridEdit.Sections(1).SectionImage = Image.FromFile(Path.GetFullPath(Path.Combine(sLocDir, "..\\..\\misc\\")) + "Icon_E.png")
        Else
            MegaGridEdit.AllowSections = False

            'Set Rows Count
            MegaGridEdit.RowsCount = rowsArr.Count
        End If

        'Configure MaskEdit TextBox for Latitude
        MegaGridEdit.ConfigureCellMaskedTextBox(-1, _
                                                1, _
                                                True, _
                                                MaskFormat.IncludeLiterals, _
                                                False, _
                                                True, _
                                                "0000'N", _
                                                Global.Microsoft.VisualBasic.ChrW(0), _
                                                Global.Microsoft.VisualBasic.ChrW(95), _
                                               False, _
                                               False, _
                                               True, _
                                               RightToLeft.No, _
                                               HorizontalAlignment.Left, _
                                               MaskFormat.IncludeLiterals, _
                                               False)

        'Configure MaskEdit TextBox for Longitude
        MegaGridEdit.ConfigureCellMaskedTextBox(-1, _
                                                2, _
                                                True, _
                                                MaskFormat.IncludeLiterals, _
                                                False, _
                                                True, _
                                                "0000'?", _
                                                Global.Microsoft.VisualBasic.ChrW(0), _
                                                Global.Microsoft.VisualBasic.ChrW(95), _
                                               False, _
                                               False, _
                                               True, _
                                               RightToLeft.No, _
                                               HorizontalAlignment.Left, _
                                               MaskFormat.IncludeLiterals, _
                                               False)

        'Configure countries' ComboBox
        If (chkAllowSections.Checked) Then
            'Configure Countries for North America
            Dim N_CountriesList As ArrayList = New ArrayList()
            N_CountriesList.AddRange(N_CountriesArray)
            MegaGridEdit.ConfigureCellComboBox(0, 5, N_CountriesList)

            'Configure Countries for Europe
            Dim E_CountriesList As ArrayList = New ArrayList()
            E_CountriesList.AddRange(E_CountriesArray)
            MegaGridEdit.ConfigureCellComboBox(1, 5, E_CountriesList)
        Else
            'No sections
            'Configure countries' ComboBox
            Dim CountriesList As ArrayList = New ArrayList()
            CountriesList.AddRange(CountriesArray)
            MegaGridEdit.ConfigureCellComboBox(-1, 5, CountriesList)
        End If

        'Configure DateTimePicker for Last Updated Date's Column
        Dim maxDate As DateTime = New DateTime(3000, 12, 1)
        Dim minDate As Date = New DateTime(2000, 12, 1)
        MegaGridEdit.ConfigureCellDatePicker(-1, 8, maxDate, minDate, DateTimePickerFormat.Custom, "yyyy-MM-dd")

        'Alert: The following 2 lines of code are very IMPORTANT and mandatory for Grid in edit mode 
        'to work properly.
        'Create the full grid and all sub grids
        MegaGridEdit.ConfigureAllGrids()

        'Update edit-mode columns' settings for the full grid and all sub grids 
        MegaGridEdit.UpdateEditColumns()
        'End of the alert

        If (MegaGridEdit.RowsCount = 0) Then
            MegaGridEdit.ClearGrid()
            Exit Sub
        End If

        Dim RowIndex As Integer = 0
        Dim NIndex As Integer = 0
        Dim EIndex As Integer = 0
        Dim SectionIndex As Integer
        For I = 0 To (rowsArr.Count - 1)
            Dim objRow As EditRow = rowsArr(I)

            If (chkAllowSections.Checked) Then
                If (objRow.IsNorthAmerica.CompareTo("YES") = 0) Then
                    'North America
                    N_rowsArr.Add(objRow)
                    SectionIndex = 0
                    RowIndex = NIndex
                Else
                    'Europe
                    E_rowsArr.Add(objRow)
                    SectionIndex = 1
                    RowIndex = EIndex

                End If
            Else
                SectionIndex = -1
                RowIndex = I
            End If

            MegaGridEdit.ConfigureEditRow(SectionIndex, RowIndex, objRow.Tag)

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 0, Convert.ToString(objRow.ID))

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 1, objRow.Latitude)

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 2, objRow.Longitude)

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 3, objRow.City)

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 4, objRow.ProvState)

            'Configure Country
            If (SectionIndex = -1) Then
                MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(CountriesArray, objRow.Country)))
            ElseIf (SectionIndex = 0) Then
                MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(N_CountriesArray, objRow.Country)))
            Else
                MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 5, Convert.ToString(GetCountryIndex(E_CountriesArray, objRow.Country)))
            End If

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 6, objRow.NationalFlagImage)

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 7, objRow.IsNorthAmerica.ToUpper())

            MegaGridEdit.ConfigureEditCell(SectionIndex, RowIndex, 8, objRow.LastUpdatedDate)

            If (chkAllowSections.Checked) Then
                If (objRow.IsNorthAmerica.CompareTo("YES") = 0) Then
                    NIndex = NIndex + 1
                Else
                    EIndex = EIndex + 1
                End If
            End If
        Next I

    End Sub

    Private Function GetCountryIndex(ByVal CArray() As String, ByVal CountryName As String) As Integer
        Dim I As Integer
        For I = 0 To (CArray.Length - 1)
            If (CountryName = CArray(I)) Then
                GetCountryIndex = I
                Exit Function
            End If
        Next I

        GetCountryIndex = -1
    End Function

    Private Function IsValidMaskedValue(ByVal sOrigStr As String)
        Try
            Dim Char_1 As Char = sOrigStr.Substring(0, 1)
            Dim Char_2 As Char = sOrigStr.Substring(1, 1)
            Dim Char_3 As Char = sOrigStr.Substring(3, 1)
            Dim Char_4 As Char = sOrigStr.Substring(4, 1)

            IsValidMaskedValue = Char.IsDigit(Char_1) And Char.IsDigit(Char_2) And Char.IsDigit(Char_3) And Char.IsDigit(Char_4)
        Catch ex As Exception
            IsValidMaskedValue = False
        End Try
    End Function

    Private Function CheckError(ByVal objEEGR As EventEditGridRow) As String
        Dim sErrorMsg As String
        Dim cellsArr As ArrayList
        Dim sCellValue As String
        Dim LastUpdatedDate As Date

        Try
            sErrorMsg = ""
            cellsArr = objEEGR.CellsList
            Dim columnsCount As Integer = cellsArr.Count
            Dim K As Integer
            For K = 0 To (columnsCount - 1)
                Dim objEditCell As EventEditGridCell = DirectCast(cellsArr(K), EventEditGridCell)
                sCellValue = objEditCell.NewValue
                If (sCellValue = Nothing) Then
                    sCellValue = ""
                End If

                If ((objEditCell.ColumnName = "Latitude") And (objEditCell.OrigColumnIndex = 1)) Then
                    'Check if Latitude value is valid or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "Latitude value is empty."
                    Else
                        If (Not IsValidMaskedValue(sCellValue)) Then
                            sErrorMsg = sErrorMsg + sCellValue + " is not a valid latitude value." + Environment.NewLine
                        End If
                    End If
                ElseIf ((objEditCell.ColumnName = "Longitude") And (objEditCell.OrigColumnIndex = 2)) Then
                    'Check if Longitude value is valid or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "Longitude value is empty."
                    Else
                        Dim sLastChar = sCellValue.Substring(sCellValue.Length - 1, 1)
                        If ((Not IsValidMaskedValue(sCellValue)) Or _
                            ((Convert.ToChar(sLastChar) <> "E"c) And (Convert.ToChar(sLastChar) <> "W"c))) Then
                            sErrorMsg = sErrorMsg + sCellValue + " is not a valid longitude value." + Environment.NewLine
                        End If
                    End If
                ElseIf ((objEditCell.ColumnName = "City") And (objEditCell.OrigColumnIndex = 3)) Then
                    'Check if City is empty or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "City is empty." + Environment.NewLine
                    End If
                ElseIf ((objEditCell.ColumnName = "Province/State") And (objEditCell.OrigColumnIndex = 4)) Then
                    'Check if Province/State is empty or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "Province/State is empty." + Environment.NewLine
                    End If
                ElseIf ((objEditCell.ColumnName = "Country") And (objEditCell.OrigColumnIndex = 5)) Then
                    'Check if Country is valid or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "Country is empty." + Environment.NewLine
                    End If
                ElseIf ((objEditCell.ColumnName = "In North America?") And (objEditCell.OrigColumnIndex = 7)) Then
                    'Check if [In North America?] is valid
                    Dim locSubMsg As String = CheckContinent(objEEGR, sCellValue.Trim())
                    If (locSubMsg.Length > 0) Then
                        sErrorMsg = locSubMsg + Environment.NewLine
                    End If
                ElseIf ((objEditCell.ColumnName = "Last Updated Date") And (objEditCell.OrigColumnIndex = 8)) Then
                    'Check if [Last Updated Date] is valid or not
                    If (sCellValue.Trim.Length = 0) Then
                        sErrorMsg = sErrorMsg + "Last Updated Date is empty." + Environment.NewLine
                    Else
                        Try
                            LastUpdatedDate = Date.ParseExact(sCellValue, "yyyy-MM-dd", Nothing)
                            If (LastUpdatedDate > Now()) Then
                                sErrorMsg = sErrorMsg + "Last Updated Date can not be greater than today." + Environment.NewLine
                            End If
                        Catch ex_1 As Exception
                            sErrorMsg = sErrorMsg + "Last Updated Date is invalid." + Environment.NewLine
                        End Try
                    End If
                End If
            Next K

            CheckError = sErrorMsg

        Catch ex_2 As Exception
            CheckError = sErrorMsg
        End Try

    End Function

    Private Sub FillEditRow(ByVal objEEGR As EventEditGridRow, ByVal objRow As EditRow)
        Dim sCellValue As String
        Dim cellsArr As ArrayList = objEEGR.CellsList
        Dim columnsCount As Integer = cellsArr.Count
        Dim K As Integer
        For K = 0 To (columnsCount - 1)
            Dim objEditCell As EventEditGridCell = DirectCast(cellsArr(K), EventEditGridCell)
            sCellValue = objEditCell.NewValue

            If ((objEditCell.ColumnName = "Latitude") And (objEditCell.OrigColumnIndex = 1)) Then
                'Latitude value
                objRow.Latitude = sCellValue
            ElseIf ((objEditCell.ColumnName = "Longitude") And (objEditCell.OrigColumnIndex = 2)) Then
                'Longitude value
                objRow.Longitude = sCellValue
            ElseIf ((objEditCell.ColumnName = "City") And (objEditCell.OrigColumnIndex = 3)) Then
                'City 
                objRow.City = sCellValue
            ElseIf ((objEditCell.ColumnName = "Province/State") And (objEditCell.OrigColumnIndex = 4)) Then
                'Province/State
                objRow.ProvState = sCellValue
            ElseIf ((objEditCell.ColumnName = "Country") And (objEditCell.OrigColumnIndex = 5)) Then
                'Country
                objRow.Country = sCellValue
            ElseIf ((objEditCell.ColumnName = "National Flag") And (objEditCell.OrigColumnIndex = 6)) Then
                'National Flag Picture
                objRow.NationalFlagImage = sCellValue
            ElseIf ((objEditCell.ColumnName = "North America?") And (objEditCell.OrigColumnIndex = 7)) Then
                'In North America?
                objRow.IsNorthAmerica = sCellValue
            ElseIf ((objEditCell.ColumnName = "Last Updated Date") And (objEditCell.OrigColumnIndex = 8)) Then
                'Last Updated Date
                objRow.LastUpdatedDate = sCellValue
            End If
        Next K
    End Sub

    Private Function CheckContinent(ByVal objEEGR As EventEditGridRow, ByVal sCellValue As String) As String
        Dim sMsg = ""
        If (objEEGR.GridIndex = -1) Then
            If (((objEEGR.SectionIndex = 0) And (sCellValue = "NO")) Or _
                ((objEEGR.SectionIndex = 1) And (sCellValue = "YES"))) Then
                sMsg = "Invalid value for [In North America?]"
            End If
        Else
            If (chkAllowSections.Checked) Then
                If (((objEEGR.GridIndex = 0) And (sCellValue = "NO")) Or _
                    ((objEEGR.GridIndex = 1) And (sCellValue = "YES"))) Then
                    sMsg = "Invalid value for [In North America?]"
                End If
            End If
        End If

        CheckContinent = sMsg
    End Function

    Private Function AddRow(ByVal objEEGR As EventEditGridRow) As Boolean
        'Insert a new record
        If (objEEGR.OnBorder) Then
            objEEGR.UseNextCopyGrid = True
        End If

        Dim cellsArr As ArrayList

        cellsArr = objEEGR.CellsList
        Dim columnsCount As Integer = cellsArr.Count
        Dim K As Integer
        For K = 0 To (columnsCount - 1)
            Dim objEditCell As EventEditGridCell = DirectCast(cellsArr(K), EventEditGridCell)
            If ((objEditCell.ColumnName = "ID") And (objEditCell.OrigColumnIndex = 0)) Then
                objEditCell.NewValue = Convert.ToString(MaxID + 1)
            End If
        Next K

        Dim newEditRow As EditRow = New EditRow()
        FillEditRow(objEEGR, newEditRow)

        'Set ID
        newEditRow.ID = MaxID + 1
        MaxID = MaxID + 1

        'Set tags
        newEditRow.Tag = MaxTag
        objEEGR.Tag = MaxTag

        If (chkAllowSections.Checked) Then
            If (objEEGR.GridIndex = -1) Then
                'In Full Grid
                If (objEEGR.SectionIndex = 0) Then
                    'North America's Section
                    If ((objEEGR.MainRowIndex >= 0) And (objEEGR.MainRowIndex < N_rowsArr.Count)) Then
                        N_rowsArr.Insert(objEEGR.MainRowIndex, newEditRow)
                    Else
                        N_rowsArr.Add(newEditRow)
                    End If
                Else
                    'Europe's Section
                    If ((objEEGR.MainRowIndex >= 0) And (objEEGR.MainRowIndex < E_rowsArr.Count)) Then
                        E_rowsArr.Insert(objEEGR.MainRowIndex, newEditRow)
                    Else
                        E_rowsArr.Add(newEditRow)
                    End If
                End If
            ElseIf (objEEGR.GridIndex = 0) Then
                'Sub Grid for North America
                If ((objEEGR.RowIndex >= 0) And (objEEGR.RowIndex < N_rowsArr.Count)) Then
                    N_rowsArr.Insert(objEEGR.RowIndex, newEditRow)
                Else
                    N_rowsArr.Add(newEditRow)
                End If
            Else
                'Sub Grid for Europe
                If ((objEEGR.RowIndex >= 0) And (objEEGR.RowIndex < E_rowsArr.Count)) Then
                    E_rowsArr.Insert(objEEGR.RowIndex, newEditRow)
                Else
                    E_rowsArr.Add(newEditRow)
                End If
            End If
        Else
            'Plain grid without section(s)
            If ((objEEGR.MainRowIndex >= 0) And (objEEGR.MainRowIndex < rowsArr.Count)) Then
                rowsArr.Insert(objEEGR.MainRowIndex, newEditRow)
            Else
                rowsArr.Add(newEditRow)
            End If
        End If

        SaveRows()

        'Bump up MaxTag by 1
        MaxTag = MaxTag + 1

        AddRow = True
    End Function

    Private Function UpdateRow(ByVal objEEGR As EventEditGridRow)
        Dim RowPos As Integer
        Dim objRow As EditRow = SearchRow(objEEGR, RowPos)

        If (RowPos = -1) Then
            UpdateRow = False
        Else
            FillEditRow(objEEGR, objRow)
            SaveRows()
            UpdateRow = True
        End If
    End Function

    Private Function DeleteRow(ByVal objEEGR As EventEditGridRow)
        Dim RowPos As Integer
        Dim objRow As EditRow = SearchRow(objEEGR, RowPos)

        If (RowPos = -1) Then
            DeleteRow = False
        Else
            Dim editRows As ArrayList = rowsArr
            If (chkAllowSections.Checked) Then
                If (objEEGR.GridIndex = -1) Then
                    'In full grid
                    If (objEEGR.SectionIndex = 0) Then
                        editRows = N_rowsArr
                    Else
                        editRows = E_rowsArr
                    End If
                Else
                    'In a sub grid
                    If (objEEGR.GridIndex = 0) Then
                        editRows = N_rowsArr
                    Else
                        editRows = E_rowsArr
                    End If
                End If
            End If

            editRows.RemoveAt(RowPos)

            SaveRows()

            DeleteRow = True
        End If
    End Function

    Private Function SearchRowInList(ByVal editRows As ArrayList, ByVal Tag As Object, ByRef RowPos As Integer) As EditRow
        Dim iCount As Integer = editRows.Count
        Dim K As Integer

        For K = 0 To (iCount - 1)
            Dim objRow As EditRow = editRows(K)
            If (objRow.Tag = Tag) Then
                SearchRowInList = objRow
                RowPos = K
                Exit Function
            End If
        Next K

        RowPos = -1
        SearchRowInList = Nothing
    End Function

    Private Function SearchRow(ByVal objEEGR As EventEditGridRow, ByRef RowPos As Integer) As EditRow
        If (chkAllowSections.Checked) Then
            Dim editRows As ArrayList
            If (objEEGR.GridIndex = -1) Then
                'In full grid
                If (objEEGR.SectionIndex = 0) Then
                    'North America's section 
                    editRows = N_rowsArr
                Else
                    'Europe's section
                    editRows = E_rowsArr
                End If
            Else
                'In a sub grid
                If (objEEGR.GridIndex = 0) Then
                    'North America's sub grid
                    editRows = N_rowsArr
                Else
                    'Europe's sub grid
                    editRows = E_rowsArr
                End If
            End If

            'Conduct search
            SearchRow = SearchRowInList(editRows, objEEGR.Tag, RowPos)
        Else
            'No sections (plain grid)
            SearchRow = SearchRowInList(rowsArr, objEEGR.Tag, RowPos)
        End If
    End Function

    Private Function RemoveBars(ByVal sOrigStr As String) As String
        RemoveBars = sOrigStr.Replace("|", "")
    End Function

    Private Sub CollectRowsInList(ByVal editRows As ArrayList, ByRef sLines As String)
        Dim sSingleLine As String
        Dim I As Integer

        Const MAX_PATH = 256
        sLines = sHeaderRow + Environment.NewLine
        For I = 0 To (rowsArr.Count - 1)
            Dim objRow As EditRow = rowsArr(I)

            sSingleLine = Convert.ToString(objRow.ID)
            sSingleLine = sSingleLine + "|" + objRow.Latitude
            sSingleLine = sSingleLine + "|" + objRow.Longitude
            sSingleLine = sSingleLine + "|" + RemoveBars(objRow.City)
            sSingleLine = sSingleLine + "|" + RemoveBars(objRow.ProvState)
            sSingleLine = sSingleLine + "|" + Convert.ToString(objRow.Country)

            Try
                Dim pszPath As StringBuilder = New StringBuilder(MAX_PATH)
                Dim pszFrom As String = sLocDir

                PathRelativePathTo(pszPath, pszFrom, FileAttributes.Directory, objRow.NationalFlagImage, FileAttributes.Normal)
                sSingleLine = sSingleLine + "|" + pszPath.ToString()
            Catch ex As Exception
                sSingleLine = sSingleLine + "|" + objRow.NationalFlagImage
            End Try

            sSingleLine = sSingleLine + "|" + objRow.IsNorthAmerica
            sSingleLine = sSingleLine + "|" + objRow.LastUpdatedDate + Environment.NewLine

            sLines = sLines + sSingleLine
        Next I

        Try
            File.WriteAllText(sFileName, sLines)
        Catch ex As Exception
        End Try

    End Sub

    Private Sub SaveRows()
        'Header
        Dim sLines As String = sHeaderRow + Environment.NewLine

        If (chkAllowSections.Checked) Then
            'Collect all rows for North America
            CollectRowsInList(N_rowsArr, sLines)

            'Collect all rows for Europe
            CollectRowsInList(E_rowsArr, sLines)
        Else
            'Plain grid
            CollectRowsInList(rowsArr, sLines)
        End If

        Try
            File.WriteAllText(sFileName, sLines)
        Catch ex As Exception
        End Try
    End Sub

    Private Sub MegaGridEdit_MGEditChangeRows(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.EditChangeRowsEventArgs) Handles MegaGridEdit.MGEditChangeRows
        Dim I As Integer
        Dim count As Integer
        Dim sSingleErrorMsg As String
        Dim sRowFlag As String
        Dim sComboFlag As String

        count = e.RowsList.Count
        For I = 0 To (count - 1)
            Dim objEEGR As EventEditGridRow
            objEEGR = DirectCast(e.RowsList(I), EventEditGridRow)
            sRowFlag = "Row with index " + Convert.ToString(objEEGR.RowIndex)
            sComboFlag = "Row with index " + Convert.ToString(objEEGR.RowIndex) + " and tag " + Convert.ToString(objEEGR.Tag)

            If (objEEGR.Flag = 1) Or (objEEGR.Flag = 2) Then
                'Add or update a row
                sSingleErrorMsg = CheckError(objEEGR)
                If (sSingleErrorMsg <> "") Then
                    objEEGR.Success = False
                    If (objEEGR.Flag = 1) Then
                        objEEGR.Message = sRowFlag + " - " + sSingleErrorMsg
                    Else
                        objEEGR.Message = sComboFlag + " - " + sSingleErrorMsg
                    End If
                Else
                    If (objEEGR.Flag = 1) Then
                        'Add a row
                        AddRow(objEEGR)
                        objEEGR.Success = True
                    Else
                        'Update a row
                        If (Not UpdateRow(objEEGR)) Then
                            objEEGR.Success = False
                            objEEGR.Message = sComboFlag + " - Can't update a row due to invalid row tag"
                        Else
                            objEEGR.Success = True
                        End If
                    End If
                End If
            Else
                'Delete a row
                If (Not DeleteRow(objEEGR)) Then
                    objEEGR.Success = False
                    objEEGR.Message = sComboFlag + " - Can't delete a row due to invalid row tag"
                Else
                    objEEGR.Success = True
                End If
            End If
        Next I

        e.Done = True
    End Sub
End Class