using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace Demo
{
    public partial class MainForm : Form
    {
		private bool bShowTopToolbar = true;
		private bool bShowAnnotationIcon = true;
		private int  mMaxTag = 2000;

        public MainForm()
        {
            InitializeComponent();
        }

		private void MainForm_Shown(object sender, EventArgs e)
		{
			cbxGridLayout.SelectedIndex = 0;
		}

		private void HandleCheckBox(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void cbxGridLayout_SelectedIndexChanged(object sender, EventArgs e)
		{
			PopulateData();
		}

		private void btnClose_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void PopulateData()
		{
			//Read the data from a file
			String locDir = System.AppDomain.CurrentDomain.BaseDirectory;
			locDir = Path.GetFullPath(Path.Combine(locDir, "..\\..\\misc\\"));
			string[] readText = File.ReadAllLines(locDir + "Cities.txt", Encoding.Default);
			int counter = 0;
			ArrayList rowsArr = new ArrayList();
			mMaxTag = 100;
			foreach (string line in readText)
			{
				if (counter > 0)
				{
					string[] cells	= line.Split('|');
					Row objRow		= new Row();
					objRow.Tag		= mMaxTag;
					objRow.CellsArr = cells;
					rowsArr.Add(objRow);

					mMaxTag++;
				}

				counter++;
			}

			MegaGrid.PopupMenu = this.cmsPopup;

			//Change the settings for MegaGrid
			ConfigureGridLayout.SetLayout(MegaGrid, cbxGridLayout.SelectedIndex);

			MegaGrid.AllowCheckBox			= chkShowCheckBox.Checked;
			MegaGrid.AllowSections			= chkAllowSections.Checked;
			MegaGrid.MoveColumns			= chkMoveColumns.Checked;
			MegaGrid.ResizeColumns			= chkResizeColumns.Checked;
			MegaGrid.SelectMultiCells		= chkMutSel.Checked;
			MegaGrid.SelectRow				= chkSelectRow.Checked;
			MegaGrid.AllowHeaderGrouping	= chkGroupColumns.Checked;
			MegaGrid.AllowZebras			= chkShadeRows.Checked;

			btnGetCheckedRows.Enabled		= chkShowCheckBox.Checked;

			if (chkFixedCol.Checked)
				MegaGrid.FixedColumns = 1;
			else
				MegaGrid.FixedColumns = 0;

			if (chkAllowSections.Checked)
			{
				chkFixedRow.Checked = false;
				chkFixedRow.Enabled = false;
			}
			else
			{
				chkFixedRow.Enabled = true;
			}

			if (chkFixedRow.Checked)
				MegaGrid.FixedRows = 1;
			else
				MegaGrid.FixedRows = 0;

			if (chkGroupColumns.Checked)
				MegaGrid.HeaderHeight = 36;
			else
				MegaGrid.HeaderHeight = 22;
			
			if (chkMutLines.Checked)
			{
				MegaGrid.Columns[4].ColumnTitle_2 = "Nickname";
				MegaGrid.HeaderHeight = 36;
				MegaGrid.RowHeight = 36;
			}
			else
			{
				MegaGrid.Columns[4].ColumnTitle_2 = "";
				if (!chkGroupColumns.Checked)
					MegaGrid.HeaderHeight = 22;
				MegaGrid.RowHeight = 22;
			}

			if (chkShowCheckBox.Checked)
			{
				chkThickCheckLine.Enabled = true;
			}
			else
			{
				chkThickCheckLine.Enabled = false;
				chkThickCheckLine.Checked = false;
			}

			if (chkThickCheckLine.Checked)
				MegaGrid.CheckLineWidth = 2;
			else
				MegaGrid.CheckLineWidth = 1;

			if (chkThickElasticCord.Checked)
				MegaGrid.ElasticCordWidth = 2;
			else
				MegaGrid.ElasticCordWidth = 1;

			int iBackColor = 0;
			int iForeColor = 0;
			if (chkSpecialColors.Checked)
			{
				MegaGrid.Columns[3].UseDefaultBackColor = false;
				MegaGrid.Columns[3].UseDefaultForeColor = false;

				MegaGrid.Columns[3].BackColor = Color.Coral;
				MegaGrid.Columns[3].ForeColor = Color.White;
			}
			else
			{
				MegaGrid.Columns[3].UseDefaultBackColor = true;
				MegaGrid.Columns[3].UseDefaultForeColor = true;
			}

			string sLine_2 = "";
			bool bIsLinkText_2 = false;
			
			MegaGrid.ConfigureAllGrids();

			if (chkAllowSections.Checked)
			{
				chkShowSectionIcon.Enabled = true;

				MegaGrid.AllowSectionIcon = chkShowSectionIcon.Checked;

				if (chkShowSectionIcon.Checked)
				{
					MegaGrid.SectionIconHeight = 16;
					MegaGrid.SectionIconWidth = 16;
				}

				//Find the number of rows for North American
				int count = 0;
				for (int i = 0; i < rowsArr.Count; i++)
				{
					Row objRow = (Row)rowsArr[i];
					string[] cells = objRow.CellsArr;
					if (cells.Length < 7)
						continue;
					string continent = cells[6];
					continent = continent.Trim();
					if (continent.CompareTo("North America") == 0)
						count++;
				}
				MegaGrid.Sections[0].RowsCount = count;

				if (chkShowSectionIcon.Checked)
					MegaGrid.Sections[0].SectionImage = Image.FromFile(locDir + "Icon_N.png");
				else
					MegaGrid.Sections[0].SectionImage = null;

				//Find the number of rows for Europe
				count = 0;
				for (int i = 0; i < rowsArr.Count; i++)
				{
					Row objRow = (Row)rowsArr[i];
					string[] cells = objRow.CellsArr;
					if (cells.Length < 7)
						continue;
					string continent = cells[6];
					continent = continent.Trim();
					if (continent.CompareTo("Europe") == 0)
						count++;
				}
				MegaGrid.Sections[1].RowsCount = count;

				if (chkShowSectionIcon.Checked)
					MegaGrid.Sections[1].SectionImage = Image.FromFile(locDir + "Icon_E.png");
				else
					MegaGrid.Sections[1].SectionImage = null;

				//Configure all cells 
				int NIndex = 0;
				int EIndex = 0;
				int SectIndex = 0;
				int RowIndex = 0;
				for (int i = 0; i < rowsArr.Count; i++)
				{
					Row objRow = (Row)rowsArr[i];
					string[] cells = objRow.CellsArr;
					if (cells.Length < 7)
						continue;

					string continent = cells[6];
					continent = continent.Trim();
					if (continent.CompareTo("North America") == 0)
					{
						SectIndex = 0;
						RowIndex = NIndex;
					}
					else
					{
						SectIndex = 1;
						RowIndex = EIndex;
					}

					MegaGrid.ConfigureRow(SectIndex, RowIndex, mMaxTag++, false);

					for (int j = 0; j < (cells.Length - 1); j++)
					{
						string strVal = cells[j];
						strVal = strVal.Trim();
						sLine_2 = "";

						if (chkSpecialColors.Checked && (j == 3))
						{
							iBackColor = MegaGrid.Columns[3].BackColor.ToArgb();
							iForeColor = MegaGrid.Columns[3].ForeColor.ToArgb();
						}
						else
						{
							iBackColor = 0;
							iForeColor = 0;
						}

						if (chkMutLines.Checked && (j == 4))
						{
							//Add the 2nd line
							sLine_2 = "N/A";
							bIsLinkText_2 = false;
							if (strVal.CompareTo("Alaska") == 0)
							{
								sLine_2 = "Last Frontier";
								bIsLinkText_2 = true;
							}
							else if (strVal.CompareTo("Nunavut") == 0)
							{
								sLine_2 = "The Land of the Midnight Sun";
								bIsLinkText_2 = true;
							}
						}
						MegaGrid.ConfigureCellText(-1,
													SectIndex,
													RowIndex,
													j,
													strVal,
													false,
													sLine_2,
													bIsLinkText_2,
													"",
													false,
													"",
													false,
													iBackColor,
													iForeColor,
													-1);

						if (j == 5)
						{
							//Display the image for Country column
							MegaGrid.ConfigureCellImage(-1,
														SectIndex,
														RowIndex,
														j,
														0,
														locDir + strVal + ".bmp",
														20,
														15,
														5);
						}
					}

					if (continent.CompareTo("North America") == 0)
						NIndex++;
					else
						EIndex++;
				}
			}
			else
			{
				//Simple layout without sections
				chkShowSectionIcon.Checked = false;
				chkShowSectionIcon.Enabled = false;
				MegaGrid.RowsCount = rowsArr.Count;

				//Configure all cells
				for (int i = 0; i < rowsArr.Count; i++)
				{
					Row objRow = (Row)rowsArr[i];
					string[] cells = objRow.CellsArr;
					if (cells.Length < 7)
						continue;

					MegaGrid.ConfigureRow(-1, i, mMaxTag++, false);
					for (int j = 0; j < (cells.Length - 1); j++)
					{
						string strVal = cells[j];
						strVal = strVal.Trim();
						sLine_2 = "";

						if (chkSpecialColors.Checked && (j == 3))
						{
							iBackColor = MegaGrid.Columns[3].BackColor.ToArgb();
							iForeColor = MegaGrid.Columns[3].ForeColor.ToArgb();
						}
						else
						{
							iBackColor = 0;
							iForeColor = 0;
						}

						if (chkMutLines.Checked && (j == 4))
						{
							//Add the 2nd line
							sLine_2 = "N/A";
							bIsLinkText_2 = false;
							if (strVal.CompareTo("Alaska") == 0)
							{
								sLine_2 = "Last Frontier";
								bIsLinkText_2 = true;
							}
							else if (strVal.CompareTo("Nunavut") == 0)
							{
								sLine_2 = "The Land of the Midnight Sun";
								bIsLinkText_2 = true;
							}
						}

						MegaGrid.ConfigureCellText(-1,
													-1,
													i,
													j,
													strVal,
													false,
													sLine_2,
													bIsLinkText_2,
													"",
													false,
													"",
													false,
													iBackColor,
													iForeColor,
													-1);

						if (j == 5)
						{
							//Display the image for Country column
							MegaGrid.ConfigureCellImage(-1,
														-1,
														i,
														j,
														0,
														locDir + strVal + ".bmp",
														20,
														15,
														5);
						}
					}
				}
			}

			MegaGrid.ActivateIcon(6);
		}

		private void MegaGrid_MGChangeGrid(object s, NET.MegaGrid.EArgs.ChangeGridEventArgs e)
		{
			String InfoMsg = "Switch to grid with index " + Convert.ToString(e.CurrGridIndex);
			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGChangeSearchMatch(object s, NET.MegaGrid.EArgs.ChangeSearchMatchEventArgs e)
		{
			String InfoMsg = "Switch to search match with index " + Convert.ToString(e.CurrMatchIndex);
			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGClickCheckBox(object s, NET.MegaGrid.CheckBoxEventArgs e)
		{
			//Add your code to handle the event of clicking a check box.
			String InfoMsg = "";

			if ((e.SectionIndex == -1) && (e.CurrRowIndex == -1))
			{
				if (e.NewValue)
					//Checked
					InfoMsg = "All rows are checked.";
				else
					//Not checked
					InfoMsg = "All rows are unchecked.";
			}
			else
			{
				if ((e.CurrRowIndex == -1) && chkAllowSections.Checked)
				{
					if (e.NewValue)
						//Checked
						InfoMsg = "All rows under section " + Convert.ToString(e.SectionIndex) + " are checked.";
					else
						//Not checked
						InfoMsg = "All rows under section " + Convert.ToString(e.SectionIndex) + " are unchecked.";
				}
				else
				{
					if (e.NewValue)
						//Checked
						InfoMsg = "Row is checked at";
					else
						//Not checked
						InfoMsg = "Row is unchecked at";

					if (e.SectionIndex >= 0)
						InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - ";

					if (chkSelectRow.Checked)
						InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + ".";
					else
						InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + ".";
				}
			}

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGClickIcon(object s, NET.MegaGrid.EArgs.ClickIconEventArgs e)
		{
			String InfoMsg = "Icon with index " + Convert.ToString(e.IconFlag) + " was clicked.";
			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGClickLink(object s, NET.MegaGrid.ClickLinkEventArgs e)
		{
			//Add your code to handle the event of clicking a link.
			String InfoMsg = "Click " + e.LinkStr + " at ";
			if (e.SectionIndex >= 0)
				InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - ";

			if (chkSelectRow.Checked)
			{
				InfoMsg += " row " + Convert.ToString(e.CurrRowIndex);
			}
			else
			{
				InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + " - ";
				InfoMsg += " column " + Convert.ToString(e.CurrColumnIndex);
			}

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGClickSectionNode(object s, NET.MegaGrid.NodeEventArgs e)
		{
			//Add your code to handle the event of clicking a section node (expand or collapse).
			String InfoMsg = "";

			if (e.SectionIndex >= 0)
			{
				if (e.NewValue)
					//Expanded
					InfoMsg = "Section " + Convert.ToString(e.SectionIndex) + " is expanded.";
				else
					//Collapsed
					InfoMsg = "Section " + Convert.ToString(e.SectionIndex) + " is collapsed.";
			}
			else
			{
				if (e.NewValue)
					//Expanded
					InfoMsg = "All sections are expanded.";
				else
					//Collapsed
					InfoMsg = "All sections are collapsed.";
			}

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGCompareRows(object s, NET.MegaGrid.CompareRowsEventArgs e)
		{
			if (e.LeftString.CompareTo(e.RightString) < 0)
				e.Result = -1;
			else
				e.Result = 1;

			e.Done = true;
		}

		private void MegaGrid_MGExpandGrid(object s, NET.MegaGrid.EArgs.ExpandGridEventArgs e)
		{
			String InfoMsg = "Grid with title " + e.GridTitle + " and index " + Convert.ToString(e.GridIndex) + " was expanded";
			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGMoveColumn(object s, NET.MegaGrid.MoveColumnEventArgs e)
		{
			String InfoMsg = "";
			if (e.SrcStartColumnIndex != e.SrcEndColumnIndex)
				InfoMsg = "Move the columns from " + Convert.ToString(e.SrcStartColumnIndex) + " to " + Convert.ToString(e.SrcEndColumnIndex);
			else
				InfoMsg = "Move the column " + Convert.ToString(e.SrcStartColumnIndex) + " to " + Convert.ToString(e.DstColumnIndex);

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGResizeColumn(object s, NET.MegaGrid.ResizeColumnEventArgs e)
		{
			String InfoMsg = "";
			if (e.RightColumnIndex == -1)
				InfoMsg = "Resize the column:" + Convert.ToString(e.LeftColumnIndex);
			else
				InfoMsg = "Resize the columns:" + Convert.ToString(e.LeftColumnIndex) + " and " + Convert.ToString(e.RightColumnIndex);

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGSelectCell(object s, NET.MegaGrid.SelectCellEventArgs e)
		{
			String InfoMsg = "Select cell at";
			if (e.SectionIndex >= 0)
				InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - ";

			if (chkSelectRow.Checked)
			{
				InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + " [row tag: " + Convert.ToString(e.RowTag) + "]";
			}
			else
			{
				InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + " [row tag: " + Convert.ToString(e.RowTag) + "]";
				InfoMsg += " column " + Convert.ToString(e.CurrColumnIndex);
			}

			txtEventInfo.Text = InfoMsg;
		}

		private void MegaGrid_MGSearchSummary(object s, NET.MegaGrid.EArgs.SearchSummaryEventArgs e)
		{
			String InfoMsg = "Total Matches: " + Convert.ToString(e.TotalMatches);
			txtEventInfo.Text = InfoMsg;
		}		

		private void btnGetCheckedRows_Click(object sender, EventArgs e)
		{
			int iGridIndex = MegaGrid.GetCurrGridIndex();
			if (iGridIndex == -2)
				return;

			int count = MegaGrid.GetCheckedRowsCount(iGridIndex);
			int SectIndex = 0;
			int RowIndex = 0;
			int OrigRowIndex = 0;

			string message = "";
			string caption = "Checked Rows List";
			for (int i = 0; i < count; i++)
			{
				MegaGrid.GetCheckedRowAt(iGridIndex, i, ref SectIndex, ref RowIndex, ref OrigRowIndex);

				if (i < 17)
				{
					//Max 17 items
					if (SectIndex >= 0)
						message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) + " [Original Row: " + Convert.ToString(OrigRowIndex) + "]\r\n";
					else
						message += "Row " + Convert.ToString(RowIndex) + " [Original Row: " + Convert.ToString(OrigRowIndex) + "}\r\n";
				}
			}

			if (message == "")
				message = "No Data";
			MessageBox.Show(message, caption, MessageBoxButtons.OK);
		}

		private void chkGetSelCells_Click(object sender, EventArgs e)
		{
			int iGridIndex = MegaGrid.GetCurrGridIndex();
			if (iGridIndex == -2)
				return;

			int count = MegaGrid.GetSelectedCellsCount(iGridIndex);
			int SectIndex = 0;
			int RowIndex = 0;
			object RowTag = new object();
			int OrigRowIndex = 0;
			int ColIndex = 0;
			int OrigColIndex = 0;

			string message = "";
			string caption = "Selected Cells List";
			for (int i = 0; i < count; i++)
			{
				MegaGrid.GetSelectedCellAt(iGridIndex, 
											i, 
											ref SectIndex, 
											ref RowIndex,
											ref OrigRowIndex,
											ref RowTag,
											ref ColIndex, 
											ref OrigColIndex);

				if (i < 17)
				{
					//Max 17 items
					if (chkSelectRow.Checked)
					{
						if (SectIndex >= 0)
							message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) + " [Original Row: " + Convert.ToString(OrigRowIndex) + "]\r\n";
						else
							message += "Row " + Convert.ToString(RowIndex) + " [Original Row: " + Convert.ToString(OrigRowIndex) + "]\r\n";
					}
					else
					{
						if (SectIndex >= 0)
							message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) +  " [Original Row: " + Convert.ToString(OrigRowIndex) + "] - Column " + Convert.ToString(ColIndex) + "\r\n";
						else
							message += "Row " + Convert.ToString(RowIndex) + " [Original Row: " + Convert.ToString(OrigRowIndex) + "] - Column " + Convert.ToString(ColIndex) + "\r\n";
					}
				}
			}

			if (message == "")
				message = "No Data";
			MessageBox.Show(message, caption, MessageBoxButtons.OK);
		}

		private void btnChangeGridTitles_Click(object sender, EventArgs e)
		{
			if (chkAllowSections.Checked)
				return;

			int iGridsCount = MegaGrid.GetTotalSubGrids();
			for (int i = 0; i < iGridsCount; i++)
			{
				MegaGrid.SetSubGridTitle(i, "Grid New Title " + Convert.ToString(i + 1));
			}
		}

		private void MegaGrid_Load(object sender, EventArgs e)
		{

		}

		private void chkToggleTopBar_CheckedChanged(object sender, EventArgs e)
		{
			bShowTopToolbar = !bShowTopToolbar;
			MegaGrid.ShowCtrl(0, bShowTopToolbar);

			if (bShowTopToolbar)
			{
				chkToggleTopBar.Text = "Hide Top Toolbar";
				chkToggleAnnot.Enabled = true;
			}
			else
			{
				chkToggleTopBar.Text = "Show Top Toolbar";
				chkToggleAnnot.Enabled = false;
			}
		}

		private void chkToggleAnnot_CheckedChanged(object sender, EventArgs e)
		{
			if (!bShowTopToolbar)
				return;

			bShowAnnotationIcon = !bShowAnnotationIcon;
			MegaGrid.ShowCtrl(1, bShowAnnotationIcon);

			if (bShowAnnotationIcon)
				chkToggleAnnot.Text = "Hide Annotation Icon";
			else
				chkToggleAnnot.Text = "Show Annotation Icon";
		}

		private void btnEditGrid_Click(object sender, EventArgs e)
		{
			EditForm objEF = new EditForm();
			objEF.ShowDialog();
			objEF.Dispose();
		}

		private void MegaGrid_MGRightClickMouse(object s, NET.MegaGrid.EArgs.ShowPopupEventArgs e)
		{
			if (e.RowIndex == 0)
				cmsPopup.Items[0].Enabled = true;
			else if (e.RowIndex == 1)
				cmsPopup.Items[1].Enabled = true;
			else
				cmsPopup.Items[2].Enabled = true;
		}

		private void MegaGrid_MGClickPopupMenuItem(object s, NET.MegaGrid.EArgs.PopupMenuItemClickedEventArgs e)
		{
			int iClickGridIndex = e.GridIndex;
			int iClickSectionIndex = e.SectionIndex;
			int iClickRowIndex = e.RowIndex;
			object iClickRowTag = e.RowTag;

			int iCopyGridIndex = -1;
			int iCopyRowIndex = -1;

			bool mUseNextCopyGrid = true;

			int iTag = Convert.ToInt16(e.Tag);
			String sText = e.Text;
			int iGridIndex = MegaGrid.GetCurrGridIndex();
			if ((sText.IndexOf("Insert") >= 0) && (iTag == 1))
			{
				//Insert a new row
				bool bResult = MegaGrid.InsertRow(iClickGridIndex, 
													iClickSectionIndex, 
													iClickRowIndex + 1,
													mMaxTag,
													mUseNextCopyGrid, 
													ref iCopyGridIndex, 
													ref iCopyRowIndex);

				if (bResult)
				{
					//Insert a cell value
					MegaGrid.ConfigureCellText(iClickGridIndex, 
												iClickSectionIndex, 
												iClickRowIndex + 1,
												0,
												Convert.ToString(mMaxTag),
												false,
												"",
												false,
												"",
												false,
												"",
												false,
												0,
												1,
												-1);

					mMaxTag++;
				}
			}
			else if ((sText.IndexOf("Update") >= 0) && (iTag == 2))
			{
				//Update a row
				MegaGrid.ConfigureCellText(iClickGridIndex,
											iClickSectionIndex,
											iClickRowIndex,
											4,
											"New Province",
											false,
											"",
											false,
											"",
											false,
											"",
											false,
											0,
											1,
											-1);
			}
			else if ((sText.IndexOf("Delete") >= 0) && (iTag == 3))
			{
				//Delete a row
				MegaGrid.DeleteRow(iClickGridIndex,
										iClickSectionIndex,
										iClickRowIndex);
			}
		}
	}
}