Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.IO
Imports System.Windows.Forms

Public Class MainForm

    Private bShowTopToolbar As Boolean = True
    Private bShowAnnotationIcon As Boolean = True
    Private rowsArr As New ArrayList()
    Private MaxTag As Integer

    Private Sub MegaGrid_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MegaGrid.Load
        cbxGridLayout.SelectedIndex = 0
    End Sub

    Private Sub cbxGridLayout_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxGridLayout.SelectedIndexChanged
        PopulateData()
    End Sub

    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        Me.Close()
    End Sub

    Private Sub PopulateData()
        'Read the data from a file
        Dim locDir As [String] = System.AppDomain.CurrentDomain.BaseDirectory
        locDir = Path.GetFullPath(Path.Combine(locDir, "..\..\misc\"))
        Dim readText As String() = File.ReadAllLines(locDir + "Cities.txt", Encoding.[Default])
        Dim counter As Integer = 0
        Dim rowsArr As New ArrayList()
        For Each line As String In readText
            If counter > 0 Then
                Dim cells As String() = line.Split("|"c)
                Dim objRow As New Row()
                objRow.CellsArr = cells
                rowsArr.Add(objRow)
            End If

            counter += 1
        Next

        MaxTag = 2000
        MegaGrid.PopupMenu = cmsPopup

        'Change the settings for MegaGrid
        ConfigureGridLayout.SetLayout(MegaGrid, cbxGridLayout.SelectedIndex)

        MegaGrid.AllowCheckBox = chkShowCheckBox.Checked
        MegaGrid.AllowSections = chkAllowSections.Checked
        MegaGrid.MoveColumns = chkMoveColumns.Checked
        MegaGrid.ResizeColumns = chkResizeColumns.Checked
        MegaGrid.SelectMultiCells = chkMutSel.Checked
        MegaGrid.SelectRow = chkSelectRow.Checked
        MegaGrid.AllowHeaderGrouping = chkGroupColumns.Checked
        MegaGrid.AllowZebras = chkShadeRows.Checked

        btnGetCheckedRows.Enabled = chkShowCheckBox.Checked

        If chkFixedCol.Checked Then
            MegaGrid.FixedColumns = 1
        Else
            MegaGrid.FixedColumns = 0
        End If

        If chkAllowSections.Checked Then
            chkFixedRow.Checked = False
            chkFixedRow.Enabled = False
        Else
            chkFixedRow.Enabled = True
        End If

        If chkFixedRow.Checked Then
            MegaGrid.FixedRows = 1
        Else
            MegaGrid.FixedRows = 0
        End If

        If chkGroupColumns.Checked Then
            MegaGrid.HeaderHeight = 36
        Else
            MegaGrid.HeaderHeight = 22
        End If

        If chkMutLines.Checked Then
            MegaGrid.Columns(4).ColumnTitle_2 = "Nickname"
            MegaGrid.HeaderHeight = 36
            MegaGrid.RowHeight = 36
        Else
            MegaGrid.Columns(4).ColumnTitle_2 = ""
            If Not chkGroupColumns.Checked Then
                MegaGrid.HeaderHeight = 22
            End If
            MegaGrid.RowHeight = 22
        End If

        If chkShowCheckBox.Checked Then
            chkThickCheckLine.Enabled = True
        Else
            chkThickCheckLine.Enabled = False
            chkThickCheckLine.Checked = False
        End If

        If chkThickCheckLine.Checked Then
            MegaGrid.CheckLineWidth = 2
        Else
            MegaGrid.CheckLineWidth = 1
        End If

        If chkThickElasticCord.Checked Then
            MegaGrid.ElasticCordWidth = 2
        Else
            MegaGrid.ElasticCordWidth = 1
        End If

        Dim iBackColor As Integer = 0
        Dim iForeColor As Integer = 0
        If chkSpecialColors.Checked Then
            MegaGrid.Columns(3).UseDefaultBackColor = False
            MegaGrid.Columns(3).UseDefaultForeColor = False

            MegaGrid.Columns(3).BackColor = Color.Coral
            MegaGrid.Columns(3).ForeColor = Color.White
        Else
            MegaGrid.Columns(3).UseDefaultBackColor = True
            MegaGrid.Columns(3).UseDefaultForeColor = True
        End If

        Dim sLine_2 As String = ""
        Dim bIsLinkText_2 As Boolean = False

        If chkAllowSections.Checked Then
            chkShowSectionIcon.Enabled = True

            MegaGrid.AllowSectionIcon = chkShowSectionIcon.Checked
            If (chkShowSectionIcon.Checked) Then
                MegaGrid.SectionIconHeight = 16
                MegaGrid.SectionIconWidth = 16
            End If

            'Find the number of rows for North American
            Dim count As Integer = 0
            For i As Integer = 0 To rowsArr.Count - 1
                Dim objRow As Row = DirectCast(rowsArr(i), Row)
                Dim cells As String() = objRow.CellsArr
                If cells.Length < 7 Then
                    Continue For
                End If
                Dim continent As String = cells(6)
                continent = continent.Trim()
                If continent.CompareTo("North America") = 0 Then
                    count += 1
                End If
            Next
            MegaGrid.Sections(0).RowsCount = count

            If (chkShowSectionIcon.Checked) Then
                MegaGrid.Sections(0).SectionImage = Image.FromFile(locDir + "Icon_N.png")
            Else
                MegaGrid.Sections(0).SectionImage = Nothing
            End If

            'Find the number of rows for Europe
            count = 0
            For i As Integer = 0 To rowsArr.Count - 1
                Dim objRow As Row = DirectCast(rowsArr(i), Row)
                Dim cells As String() = objRow.CellsArr
                If cells.Length < 7 Then
                    Continue For
                End If
                Dim continent As String = cells(6)
                continent = continent.Trim()
                If continent.CompareTo("Europe") = 0 Then
                    count += 1
                End If
            Next
            MegaGrid.Sections(1).RowsCount = count

            If (chkShowSectionIcon.Checked) Then
                MegaGrid.Sections(1).SectionImage = Image.FromFile(locDir + "Icon_E.png")
            Else
                MegaGrid.Sections(1).SectionImage = Nothing
            End If

            MegaGrid.ConfigureAllGrids()

            'Configure all cells 
            Dim NIndex As Integer = 0
            Dim EIndex As Integer = 0
            Dim SectIndex As Integer = 0
            Dim RowIndex As Integer = 0
            For i As Integer = 0 To rowsArr.Count - 1
                Dim objRow As Row = DirectCast(rowsArr(i), Row)
                Dim cells As String() = objRow.CellsArr
                If cells.Length < 7 Then
                    Continue For
                End If

                Dim continent As String = cells(6)
                continent = continent.Trim()
                If continent.CompareTo("North America") = 0 Then
                    SectIndex = 0
                    RowIndex = NIndex
                Else
                    SectIndex = 1
                    RowIndex = EIndex
                End If

                MegaGrid.ConfigureRow(SectIndex, RowIndex, MaxTag, False)
                MaxTag = MaxTag + 1

                For j As Integer = 0 To (cells.Length - 1) - 1
                    Dim strVal As String = cells(j)
                    strVal = strVal.Trim()
                    sLine_2 = ""

                    If chkSpecialColors.Checked AndAlso (j = 3) Then
                        iBackColor = MegaGrid.Columns(3).BackColor.ToArgb()
                        iForeColor = MegaGrid.Columns(3).ForeColor.ToArgb()
                    Else
                        iBackColor = 1
                        iForeColor = 1
                    End If

                    If chkMutLines.Checked AndAlso (j = 4) Then
                        'Add the 2nd line
                        sLine_2 = "N/A"
                        bIsLinkText_2 = False
                        If strVal.CompareTo("Alaska") = 0 Then
                            sLine_2 = "Last Frontier"
                            bIsLinkText_2 = True
                        ElseIf strVal.CompareTo("Nunavut") = 0 Then
                            sLine_2 = "The Land of the Midnight Sun"
                            bIsLinkText_2 = True
                        End If
                    End If
                    MegaGrid.ConfigureCellText(-1, _
                                                SectIndex, _
                                                RowIndex, _
                                                j, _
                                                strVal, _
                                                False, _
                                                sLine_2, _
                                                bIsLinkText_2, _
                                                "", _
                                                False, _
                                                "", _
                                                False, _
                                                iBackColor, _
                                                iForeColor, _
                                                -1)

                    If j = 5 Then
                        'Display the image for Country column
                        MegaGrid.ConfigureCellImage(-1, _
                                                    SectIndex, _
                                                    RowIndex, _
                                                    j, _
                                                    0, _
                                                    (locDir & strVal) + ".bmp", _
                                                    20, _
                                                    15, _
                                                    5)
                    End If
                Next

                If continent.CompareTo("North America") = 0 Then
                    NIndex += 1
                Else
                    EIndex += 1
                End If
            Next
        Else
            MegaGrid.ConfigureAllGrids()

            'Simple layout without sections
            chkShowSectionIcon.Enabled = False
            chkShowSectionIcon.Checked = False

            MegaGrid.RowsCount = rowsArr.Count

            'Configure all cells
            For i As Integer = 0 To rowsArr.Count - 1
                Dim objRow As Row = DirectCast(rowsArr(i), Row)
                Dim cells As String() = objRow.CellsArr
                If cells.Length < 7 Then
                    Continue For
                End If

                MegaGrid.ConfigureRow(-1, i, MaxTag, False)
                MaxTag = MaxTag + 1

                For j As Integer = 0 To (cells.Length - 1) - 1
                    Dim strVal As String = cells(j)
                    strVal = strVal.Trim()
                    sLine_2 = ""

                    If chkSpecialColors.Checked And (j = 3) Then
                        iBackColor = MegaGrid.Columns(3).BackColor.ToArgb()
                        iForeColor = MegaGrid.Columns(3).ForeColor.ToArgb()
                    Else
                        iBackColor = 1
                        iForeColor = 1
                    End If

                    If chkMutLines.Checked And (j = 4) Then
                        'Add the 2nd line
                        sLine_2 = "N/A"
                        bIsLinkText_2 = False
                        If strVal.CompareTo("Alaska") = 0 Then
                            sLine_2 = "Last Frontier"
                            bIsLinkText_2 = True
                        ElseIf strVal.CompareTo("Nunavut") = 0 Then
                            sLine_2 = "The Land of the Midnight Sun"
                            bIsLinkText_2 = True
                        End If
                    End If

                    MegaGrid.ConfigureCellText(-1, _
                                                -1, _
                                                i, _
                                                j, _
                                                strVal, _
                                                False, _
                                                sLine_2, _
                                                bIsLinkText_2, _
                                                "", _
                                                False, _
                                                "", _
                                                False, _
                                                iBackColor, _
                                                iForeColor, _
                                                -1)

                    If j = 5 Then
                        'Display the image for Country column
                        MegaGrid.ConfigureCellImage(-1, _
                                                    -1, _
                                                    i, _
                                                    j, _
                                                    0, _
                                                    (locDir & strVal) + ".bmp", _
                                                    20, _
                                                    15, _
                                                    5)
                    End If
                Next
            Next
        End If

        MegaGrid.ActivateIcon(6)
    End Sub

    Private Sub HandleCheckBox(ByVal sender As Object, ByVal e As EventArgs) Handles chkThickElasticCord.CheckedChanged, chkThickCheckLine.CheckedChanged, chkSpecialColors.CheckedChanged, chkShowSectionIcon.CheckedChanged, chkShowCheckBox.CheckedChanged, chkShadeRows.CheckedChanged, chkSelectRow.CheckedChanged, chkResizeColumns.CheckedChanged, chkMutSel.CheckedChanged, chkMutLines.CheckedChanged, chkMoveColumns.CheckedChanged, chkGroupColumns.CheckedChanged, chkFixedRow.CheckedChanged, chkFixedCol.CheckedChanged, chkAllowSections.CheckedChanged
        PopulateData()
    End Sub

    Private Sub MegaGrid_MGChangeGrid(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.ChangeGridEventArgs)
        Dim InfoMsg As [String] = ""
        InfoMsg = "Switch to grid with index " + Convert.ToString(e.CurrGridIndex)

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGChangeSearchMatch(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.ChangeSearchMatchEventArgs)
        Dim InfoMsg As [String] = ""
        InfoMsg = "Switch to search match with index " + Convert.ToString(e.CurrMatchIndex)

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGClickCheckBox(ByVal s As System.Object, ByVal e As NET.MegaGrid.CheckBoxEventArgs)
        'Add your code to handle the event of clicking a checkbox.
        Dim InfoMsg As [String] = ""

        If (e.SectionIndex = -1) AndAlso (e.CurrRowIndex = -1) Then
            If e.NewValue Then
                'Checked
                InfoMsg = "All rows are checked."
            Else
                'Not checked
                InfoMsg = "All rows are unchecked."
            End If
        Else
            If (e.CurrRowIndex = -1) AndAlso chkAllowSections.Checked Then
                If e.NewValue Then
                    'Checked
                    InfoMsg = "All rows under section " + Convert.ToString(e.SectionIndex) + " are checked."
                Else
                    'Not checked
                    InfoMsg = "All rows under section " + Convert.ToString(e.SectionIndex) + " are unchecked."
                End If
            Else
                If e.NewValue Then
                    'Checked
                    InfoMsg = "Row is checked at"
                Else
                    'Not checked
                    InfoMsg = "Row is unchecked at"
                End If

                If e.SectionIndex >= 0 Then
                    InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - "
                End If

                If chkSelectRow.Checked Then
                    InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + "."
                Else
                    InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + "."
                End If
            End If
        End If
    End Sub

    Private Sub MegaGrid_MGClickIcon(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.ClickIconEventArgs)
        Dim InfoMsg As [String] = ""
        InfoMsg = "Icon with index " + Convert.ToString(e.IconFlag) + " was clicked."

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGClickLink(ByVal s As System.Object, ByVal e As NET.MegaGrid.ClickLinkEventArgs)
        'Add your code to handle the event of clicking a link.
        Dim InfoMsg As [String] = "Click " + e.LinkStr + " at "
        If e.SectionIndex >= 0 Then
            InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - "
        End If

        If chkSelectRow.Checked Then
            InfoMsg += " row " + Convert.ToString(e.CurrRowIndex)
        Else
            InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + " - "
            InfoMsg += " column " + Convert.ToString(e.CurrColumnIndex)
        End If

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGClickSectionNode(ByVal s As System.Object, ByVal e As NET.MegaGrid.NodeEventArgs)
        'Add your code to handle the event of clicking a section node (expanded or collapsed).
        Dim InfoMsg As [String] = ""

        If e.SectionIndex >= 0 Then
            If e.NewValue Then
                'Expanded
                InfoMsg = "Section " + Convert.ToString(e.SectionIndex) + " is expanded."
            Else
                'Collapsed
                InfoMsg = "Section " + Convert.ToString(e.SectionIndex) + " is collapsed."
            End If
        Else
            If e.NewValue Then
                'Expanded
                InfoMsg = "All sections are expanded."
            Else
                'Collapsed
                InfoMsg = "All sections are collapsed."
            End If
        End If

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGCompareRows(ByVal s As System.Object, ByVal e As NET.MegaGrid.CompareRowsEventArgs)
        If (e.LeftString.CompareTo(e.RightString) < 0) Then
            e.Result = -1
        Else
            e.Result = 1
        End If

        e.Done = True
    End Sub

    Private Sub MegaGrid_MGExpandGrid(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.ExpandGridEventArgs)
        Dim InfoMsg As [String] = ""
        InfoMsg = "Grid with title " + e.GridTitle + " and index " + Convert.ToString(e.GridIndex) + " was expanded"

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGMoveColumn(ByVal s As System.Object, ByVal e As NET.MegaGrid.MoveColumnEventArgs)
        Dim InfoMsg As [String] = ""
        If e.SrcStartColumnIndex <> e.SrcEndColumnIndex Then
            InfoMsg = "Move the columns from " + Convert.ToString(e.SrcStartColumnIndex) + " to " + Convert.ToString(e.SrcEndColumnIndex)
        Else
            InfoMsg = "Move the column " + Convert.ToString(e.SrcStartColumnIndex) + " to " + Convert.ToString(e.DstColumnIndex)
        End If

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGResizeColumn(ByVal s As System.Object, ByVal e As NET.MegaGrid.ResizeColumnEventArgs)
        Dim InfoMsg As [String] = ""
        If e.RightColumnIndex = -1 Then
            InfoMsg = "Resize the column:" + Convert.ToString(e.LeftColumnIndex)
        Else
            InfoMsg = "Resize the columns:" + Convert.ToString(e.LeftColumnIndex) + " and " + Convert.ToString(e.RightColumnIndex)
        End If

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGSelectCell(ByVal s As System.Object, ByVal e As NET.MegaGrid.SelectCellEventArgs)
        Dim InfoMsg As [String] = "Select cell at"
        If e.SectionIndex >= 0 Then
            InfoMsg += " section " + Convert.ToString(e.SectionIndex) + " - "
        End If

        If chkSelectRow.Checked Then
            InfoMsg += " row " + Convert.ToString(e.CurrRowIndex)
        Else
            InfoMsg += " row " + Convert.ToString(e.CurrRowIndex) + " - "
            InfoMsg += " column " + Convert.ToString(e.CurrColumnIndex)
        End If

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub MegaGrid_MGSearchSummary(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.SearchSummaryEventArgs)
        Dim InfoMsg As [String] = ""
        InfoMsg = "Total Matches: " + Convert.ToString(e.TotalMatches)

        txtEventInfo.Text = InfoMsg
    End Sub

    Private Sub chkGetSelCells_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkGetSelCells.Click
        Dim GridIndex As Integer = MegaGrid.GetCurrGridIndex()
        If (GridIndex = -2) Then
            Exit Sub
        End If

        Dim count As Integer = MegaGrid.GetSelectedCellsCount(GridIndex)
        Dim SectIndex As Integer = 0
        Dim RowIndex As Integer = 0
        Dim RowTag As Object = New Object()
        Dim OrigRowIndex As Integer = 0
        Dim ColIndex As Integer = 0
        Dim OrigColIndex As Integer = 0

        Dim message As String = ""
        Dim caption As String = "Selected Cells List"
        For i As Integer = 0 To count - 1
            MegaGrid.GetSelectedCellAt(GridIndex, i, SectIndex, RowIndex, OrigRowIndex, RowTag, ColIndex, OrigColIndex)

            If i < 17 Then
                'Max 17 items
                If chkSelectRow.Checked Then
                    If SectIndex >= 0 Then
                        message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) + vbCr & vbLf
                    Else
                        message += "Row " + Convert.ToString(RowIndex) + vbCr & vbLf
                    End If
                Else
                    If SectIndex >= 0 Then
                        message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) + " - Column " + Convert.ToString(ColIndex) + vbCr & vbLf
                    Else
                        message += "Row " + Convert.ToString(RowIndex) + " - Column " + Convert.ToString(ColIndex) + vbCr & vbLf
                    End If
                End If
            End If
        Next

        If message = "" Then
            message = "No Data"
        End If
        MessageBox.Show(message, caption, MessageBoxButtons.OK)
    End Sub

    Private Sub btnGetCheckedRows_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetCheckedRows.Click
        Dim GridIndex As Integer = MegaGrid.GetCurrGridIndex()
        If (GridIndex = -2) Then
            Exit Sub
        End If

        Dim count As Integer = MegaGrid.GetCheckedRowsCount(GridIndex)
        Dim SectIndex As Integer = 0
        Dim RowIndex As Integer = 0
        Dim OrigRowIndex As Integer = 0

        Dim message As String = ""
        Dim caption As String = "Checked Rows List"
        For i As Integer = 0 To count - 1
            MegaGrid.GetCheckedRowAt(GridIndex, i, SectIndex, RowIndex, OrigRowIndex)

            If i < 17 Then
                'Max 17 items
                If SectIndex >= 0 Then
                    message += "Section " + Convert.ToString(SectIndex) + " - Row " + Convert.ToString(RowIndex) + vbCr & vbLf
                Else
                    message += "Row " + Convert.ToString(RowIndex) + vbCr & vbLf
                End If
            End If
        Next

        If message = "" Then
            message = "No Data"
        End If
        MessageBox.Show(message, caption, MessageBoxButtons.OK)
    End Sub

    Private Sub btnChangeGridTitles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnChangeGridTitles.Click
        If chkAllowSections.Checked Then
            Exit Sub
        End If

        Dim iGridsCount As Integer

        iGridsCount = MegaGrid.GetTotalSubGrids()
        For I As Integer = 0 To (iGridsCount - 1)
            MegaGrid.SetSubGridTitle(I, "Grid New Title " + Convert.ToString(I + 1))
        Next I
    End Sub

    Private Sub chkToggleTopBar_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkToggleTopBar.CheckedChanged
        bShowTopToolbar = Not bShowTopToolbar
        MegaGrid.ShowCtrl(0, bShowTopToolbar)

        If (bShowTopToolbar) Then
            chkToggleTopBar.Text = "Hide Top Toolbar"
            chkToggleAnnot.Enabled = True
        Else
            chkToggleTopBar.Text = "Show Top Toolbar"
            chkToggleAnnot.Enabled = False
        End If
    End Sub

    Private Sub chkToggleAnnot_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkToggleAnnot.CheckedChanged
        If (Not bShowTopToolbar) Then
            Exit Sub
        End If

        bShowAnnotationIcon = Not bShowAnnotationIcon
        MegaGrid.ShowCtrl(1, bShowAnnotationIcon)

        If (bShowAnnotationIcon) Then
            chkToggleAnnot.Text = "Hide Annotation Icon"
        Else
            chkToggleAnnot.Text = "Show Annotation Icon"
        End If
    End Sub

    Private Sub btnEditGrid_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEditGrid.Click
        Dim frmEditGrid As EditModeForm

        frmEditGrid = New EditModeForm()
        frmEditGrid.Show()
    End Sub

    Private Sub MegaGrid_MGRightClickMouse(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.ShowPopupEventArgs)
        Dim iClickGridIndex As Integer = e.GridIndex
        Dim iClickSectionIndex As Integer = e.SectionIndex
        Dim iClickRowIndex As Integer = e.RowIndex

        'Enable or disable menu items?
        cmsPopup.Items(0).Enabled = True
        cmsPopup.Items(1).Enabled = True
        cmsPopup.Items(2).Enabled = True

    End Sub

    Private Sub MegaGrid_MGClickPopupMenuItem(ByVal s As System.Object, ByVal e As NET.MegaGrid.EArgs.PopupMenuItemClickedEventArgs)
        Dim iClickGridIndex As Integer = e.GridIndex
        Dim iClickSectionIndex As Integer = e.SectionIndex
        Dim iClickRowIndex As Integer = e.RowIndex

        'Add a new row
        If ((e.Text = "Add Row") Or (Convert.ToInt16(e.Tag) = 1)) Then
            'Insert a new row
            Dim bResult As Boolean
            Dim iCopyGridIndex As Integer
            Dim iCopyRowIndex As Integer
            Dim iNewRowIndex = iClickRowIndex + 1
            bResult = MegaGrid.InsertRow(iClickGridIndex, _
                                        iClickSectionIndex, _
                                        iNewRowIndex, _
                                        MaxTag, _
                                        True, _
                                        iCopyGridIndex, _
                                        iCopyRowIndex)

            If (bResult) Then
                MaxTag = MaxTag + 1

                'Set the value for 1st cell inside the new row
                MegaGrid.ConfigureCellText(iClickGridIndex, _
                                          iClickSectionIndex, _
                                          iNewRowIndex, _
                                          0, _
                                          MaxTag, _
                                          False, _
                                          "", _
                                          False, _
                                          "", _
                                          False, _
                                          "", _
                                          False, _
                                          Color.White.ToArgb(), _
                                          Color.Orange.ToArgb(), _
                                          -1)
            End If
            Exit Sub
        End If

        'Update a row
        If ((e.Text = "Update Row") Or (Convert.ToInt16(e.Tag) = 2)) Then
            'Update the value for 1st cell inside the new row
            MegaGrid.ConfigureCellText(iClickGridIndex, _
                                          iClickSectionIndex, _
                                          iClickRowIndex, _
                                          3, _
                                          "New City", _
                                          False, _
                                          "", _
                                          False, _
                                          "", _
                                          False, _
                                          "", _
                                          False, _
                                          Color.White.ToArgb(), _
                                          Color.Red.ToArgb(), _
                                          -1)
            Exit Sub
        End If

        If ((e.Text = "Delete Row") Or (Convert.ToInt16(e.Tag) = 3)) Then
            'Delete a row
            MegaGrid.DeleteRow(iClickGridIndex, iClickSectionIndex, iClickRowIndex)
            Exit Sub
        End If

    End Sub

End Class
